/**
 * This script creates the 'leaguetable' TinyMCE plugin with the related 'selectleaguetable' TinyMCE button, more information
 * are available in the TinyMCE documentation:
 *
 * - https://www.tinymce.com/docs/
 *
 * The modal window is handled with the windowManager API:
 *
 * - https://www.tinymce.com/docs/api/tinymce/tinymce.windowmanager/
 *
 * To have a general idea on how this script works check out the following resources:
 *
 * - https://www.tinymce.com/docs/advanced/creating-a-plugin/
 * - http://jamesdigioia.com/add-button-pop-wordpresss-tinymce-editor/
 */
(function($) {

  //Regex
  var digitsRegex = /^\s*\d+\s*$/;
  var hexRgbRegex = /^#(?:[0-9a-fA-F]{3}){1,2}$/;
  var opacityRegex = /(^\s*\d+\s*$)|(^\s*\d\.\d{1,2}\s*$)/;

  //Creates the 'hreflangflags' plugin
  tinymce.PluginManager.add('hreflangflags', function(editor, url) {

    //Creates the 'selecthreflangflags' TinyMCE button and associate the 'openhreflangflags' command with its click
    editor.addButton('selecthreflangflags',
        {title: 'Hreflang Flags', cmd: 'openhreflangflags', icon: 'icon dashicons-flag',});

    //Create the 'openhreflangflags' command
    editor.addCommand('openhreflangflags', function() {

      //Open the modal window
      editor.windowManager.open({

        //Set the various properties of the modal window
        title: 'Hreflang Flags',
        width: 300,
        height: 546,
        id: 'dahf-hreflangflags',

        /**
         * Add buttons in the bottom of the modal window. (eg. Add Flags, Add Shortcode, Insert, Cancel, etc.)
         *
         * The buttons should be provided as objects of this array, in this case there is only a single object/button.
         */
        buttons: [
          {
            text: 'Add Flags',
            id: 'dahf-button-add-flags',
            class: 'dalt-button'
            ,
            onclick: function(e) {

              var flagsSize = $('#flags-size').val();
              var orientation = $('#orientation').val();
              var margin = $('#margin').val();
              var containerColor = $('#container-color').val();
              var containerOpacity = $('#container-opacity').val();
              var containerHorizontalPadding = $('#container-horizontal-padding').val();
              var containerVerticalPadding = $('#container-vertical-padding').val();
              var containerBorderRadius = $('#container-border-radius').val();

              //Generate the shortcode
              var shortcode = '[hreflang-flags flags-size="' + flagsSize + '"';

              //Orientation
              shortcode += ' orientation="' + orientation + '"';

              //Margin
              if (margin.match(digitsRegex)) {
                shortcode += ' margin="' + margin + '"';
              } else {
                shortcode += ' margin="5"';
              }

              //Container Color
              if (containerColor.match(hexRgbRegex)) {
                shortcode += ' container-color="' + containerColor + '"';
              } else {
                shortcode += ' container-color="#000000"';
              }

              //container Opacity
              if (containerOpacity.match(opacityRegex)) {
                shortcode += ' container-opacity="' + containerOpacity + '"';
              } else {
                shortcode += ' container-opacity="0"';
              }

              //Container Horizontal Padding
              if (containerHorizontalPadding.match(digitsRegex)) {
                shortcode += ' container-horizontal-padding="' + containerHorizontalPadding + '"';
              } else {
                shortcode += ' container-horizontal-padding="0"';
              }

              //Container Vertical Padding
              if (containerVerticalPadding.match(digitsRegex)) {
                shortcode += ' container-vertical-padding="' + containerVerticalPadding + '"';
              } else {
                shortcode += ' container-vertical-padding="0"';
              }

              //Container Border Radius
              if (containerBorderRadius.match(digitsRegex)) {
                shortcode += ' container-border-radius="' + containerBorderRadius + '"]';
              } else {
                shortcode += ' container-border-radius="0"]';
              }

              //Add the shortcode in the post
              editor.execCommand('mceInsertContent', false, shortcode);

              //Close the modal window
              editor.windowManager.close();
            },
          }],

      });

      //Generates the content of modal window
      appendContent();

    });

  });

  /**
   * Get the modal window content with an AJAX request and append it in the modal window.
   */
  function appendContent() {

    //Get a reference to the DOM element
    var contentBody = $('#dahf-hreflangflags-body');

    //Prepare the parameters for the AJAX request
    var data = {
      action: 'dahf_get_modal_window_content',
      security: dahf_nonce,
    };

    //Send the AJAX request
    $.post(dahf_ajax_url, data, function(responseHtml) {

      //Append the content in the modal window
      $(contentBody).append(responseHtml);

      //Initialize Chosen
      $('#flags-size, #orientation').chosen();

    });

  }

})(jQuery);