<?php

/*
 * this class should be used to work with the administrative side of wordpress
 */

class Dahf_Admin {

	protected static $instance = null;
	private $shared = null;
	private $screen_id_options = null;

	private function __construct() {

		//assign an instance of the plugin info
		$this->shared = Dahf_Shared::get_instance();

		//Load admin stylesheets and JavaScript
		add_action( 'admin_enqueue_scripts', array( $this, 'enqueue_admin_styles' ) );
		add_action( 'admin_enqueue_scripts', array( $this, 'enqueue_admin_scripts' ) );

		//Write in back end head
		add_action( 'admin_head', array( $this, 'wr_admin_head' ) );

		//Add the admin menu
		add_action( 'admin_menu', array( $this, 'me_add_admin_menu' ) );

		//Load the options API registrations and callbacks
		add_action( 'admin_init', array( $this, 'op_register_options' ) );

		//Add the meta box
		add_action( 'add_meta_boxes', array( $this, 'create_meta_box' ) );

		//Save the meta box
		add_action( 'save_post', array( $this, 'dahf_save_meta_interlinks_options' ) );

		//this hook is triggered during the creation of a new blog
		add_action( 'wpmu_new_blog', array( $this, 'new_blog_create_options_and_tables' ), 10, 6 );

		//this hook is triggered during the deletion of a blog
		add_action( 'delete_blog', array( $this, 'delete_blog_delete_options_and_tables' ), 10, 1 );

		//Create tinymce plugin
		add_action( 'init', array( $this, 'create_tinymce_plugin' ) );

	}

	/*
	 * return an istance of this class
	 */
	public static function get_instance() {

		if ( null == self::$instance ) {
			self::$instance = new self;
		}

		return self::$instance;

	}

	/*
	 * write in the admin head
	 */
	public function wr_admin_head() {

		echo '<script type="text/javascript">';
		echo 'var dahf_ajax_url = "' . admin_url( 'admin-ajax.php' ) . '";';
		echo 'var dahf_nonce = "' . wp_create_nonce( "dahf" ) . '";';
		echo '</script>';

	}

	public function enqueue_admin_styles() {

		$screen = get_current_screen();

		//menu options
		if ( $screen->id == $this->screen_id_options ) {
			wp_enqueue_style( $this->shared->get( 'slug' ) . '-framework-options',
				$this->shared->get( 'url' ) . 'admin/assets/css/framework/options.css', array(),
				$this->shared->get( 'ver' ) );
			wp_enqueue_style( $this->shared->get( 'slug' ) . '-jquery-ui-tooltip',
				$this->shared->get( 'url' ) . 'admin/assets/css/jquery-ui-tooltip.css', array(),
				$this->shared->get( 'ver' ) );
			wp_enqueue_style( $this->shared->get( 'slug' ) . '-chosen',
				$this->shared->get( 'url' ) . 'admin/assets/inc/chosen/chosen-min.css', array(),
				$this->shared->get( 'ver' ) );
			wp_enqueue_style( $this->shared->get( 'slug' ) . '-chosen-custom',
				$this->shared->get( 'url' ) . 'admin/assets/css/chosen-custom.css', array(),
				$this->shared->get( 'ver' ) );
			wp_enqueue_style( $this->shared->get( 'slug' ) . '-options-menu',
				$this->shared->get( 'url' ) . 'admin/assets/css/options-menu.css', array(),
				$this->shared->get( 'ver' ) );
			wp_enqueue_style( 'wp-color-picker' );
		}

		//Enqueue scripts in all the post types of the post editor
		$args               = array(
			'show_ui' => true
		);
		$post_types_with_ui = get_post_types( $args );
		unset( $post_types_with_ui['attachment'] );

		if ( in_array( $screen->id, $post_types_with_ui ) and current_user_can( 'edit_posts' ) ) {

			//Load Chosen CSS
			wp_enqueue_style( $this->shared->get( 'slug' ) . '-chosen',
				$this->shared->get( 'url' ) . 'admin/assets/inc/chosen/chosen-min.css', array(),
				$this->shared->get( 'ver' ) );

            /*
             * This enables the use of custom icons from Dashicons without using a .png image.
             *
             * For details: https://www.gavick.com/blog/wordpress-tinymce-custom-buttons#tc-section-4
             */
            wp_enqueue_style( $this->shared->get( 'slug' ) . '-enable-dashicons-in-tinymce', $this->shared->get( 'url' ) . 'admin/assets/css/enable-dashicons-in-tinymce.css', array(), $this->shared->get( 'ver' ) );

			//Load the CSS used in the modal window opened when the TinyMCE Hreflang Flags button is clicked
			wp_enqueue_style( $this->shared->get( 'slug' ) . '-tinymce-modal-window',
				$this->shared->get( 'url' ) . 'admin/assets/css/tinymce-hreflangflags.css', array(),
				$this->shared->get( 'ver' ) );

		}

	}

	/*
	 * enqueue admin-specific javascript
	 */
	public function enqueue_admin_scripts() {

		$screen = get_current_screen();

		//menu options
		if ( $screen->id == $this->screen_id_options ) {
			wp_enqueue_script( 'jquery-ui-tooltip' );
			wp_enqueue_script( $this->shared->get( 'slug' ) . '-jquery-ui-tooltip-init',
				$this->shared->get( 'url' ) . 'admin/assets/js/jquery-ui-tooltip-init.js', 'jquery',
				$this->shared->get( 'ver' ) );
			wp_enqueue_script( $this->shared->get( 'slug' ) . '-chosen',
				$this->shared->get( 'url' ) . 'admin/assets/inc/chosen/chosen-min.js', array( 'jquery' ),
				$this->shared->get( 'ver' ) );
			wp_enqueue_script( $this->shared->get( 'slug' ) . '-options-menu',
				$this->shared->get( 'url' ) . 'admin/assets/js/options-menu.js', array( 'jquery' ),
				$this->shared->get( 'ver' ) );
			wp_enqueue_script( 'wp-color-picker' );
		}

		//Enqueue scripts in all the post types of the post editor
		$args               = array(
			'show_ui' => true
		);
		$post_types_with_ui = get_post_types( $args );
		unset( $post_types_with_ui['attachment'] );

		if ( in_array( $screen->id, $post_types_with_ui ) and current_user_can( 'edit_posts' ) ) {

			//Load the Chosen JS
			wp_enqueue_script( $this->shared->get( 'slug' ) . '-chosen',
				$this->shared->get( 'url' ) . 'admin/assets/inc/chosen/chosen-min.js', 'jquery',
				$this->shared->get( 'ver' ) );

		}

	}

	/*
	 * plugin activation
	 */
	public function ac_activate( $networkwide ) {

		/*
		 * delete options and tables for all the sites in the network
		 */
		if ( function_exists( 'is_multisite' ) and is_multisite() ) {

			/*
			 * if this is a "Network Activation" create the options and tables
			 * for each blog
			 */
			if ( $networkwide ) {

				//get the current blog id
				global $wpdb;
				$current_blog = $wpdb->blogid;

				//create an array with all the blog ids
				$blogids = $wpdb->get_col( "SELECT blog_id FROM $wpdb->blogs" );

				//iterate through all the blogs
				foreach ( $blogids as $blog_id ) {

					//swith to the iterated blog
					switch_to_blog( $blog_id );

					//create options for the iterated blog
					$this->ac_initialize_options();

				}

				//switch to the current blog
				switch_to_blog( $current_blog );

			} else {

				/*
				 * if this is not a "Network Activation" create options only for the current blog
				 */
				$this->ac_initialize_options();

			}

		} else {

			/*
			 * if this is not a multisite installation create options only for the current blog
			 */
			$this->ac_initialize_options();

		}

	}

	//create the options and tables for the newly created blog
	public function new_blog_create_options_and_tables( $blog_id, $user_id, $domain, $path, $site_id, $meta ) {

		global $wpdb;

		/*
		 * if the plugin is "Network Active" create the options and tables for
		 * this new blog
		 */
		if ( is_plugin_active_for_network( 'interlinks-manager/init.php' ) ) {

			//get the id of the current blog
			$current_blog = $wpdb->blogid;

			//switch to the blog that is being activated
			switch_to_blog( $blog_id );

			//create options for the new blog
			$this->ac_initialize_options();

			//switch to the current blog
			switch_to_blog( $current_blog );

		}

	}

	//delete options and tables for the deleted blog
	public function delete_blog_delete_options_and_tables( $blog_id ) {

		global $wpdb;

		//get the id of the current blog
		$current_blog = $wpdb->blogid;

		//switch to the blog that is being activated
		switch_to_blog( $blog_id );

		//create options for the new blog
		$this->un_delete_options();

		//switch to the current blog
		switch_to_blog( $current_blog );

	}

	/*
	 * initialize plugin options
	 */
	private function ac_initialize_options() {

		//General
		add_option( $this->shared->get( 'slug' ) . '_flags_source', "language-locale" );
		add_option( $this->shared->get( 'slug' ) . '_flags_type', "shiny" );
		add_option( $this->shared->get( 'slug' ) . '_flags_initial_opacity', "0.9" );
		add_option( $this->shared->get( 'slug' ) . '_language_alt_attribute', "0" );
		add_option( $this->shared->get( 'slug' ) . '_language_alt_attribute_prefix', "" );
		add_option( $this->shared->get( 'slug' ) . '_language_alt_attribute_suffix', "" );
		add_option( $this->shared->get( 'slug' ) . '_locale_alt_attribute', "0" );
		add_option( $this->shared->get( 'slug' ) . '_locale_alt_attribute_prefix', "" );
		add_option( $this->shared->get( 'slug' ) . '_locale_alt_attribute_suffix', "" );

		//Fixed
		add_option( $this->shared->get( 'slug' ) . '_fixed_flags', "1" );
		add_option( $this->shared->get( 'slug' ) . '_fixed_flags_mobile', "1" );
		add_option( $this->shared->get( 'slug' ) . '_fixed_flags_breakpoint', "0" );
		add_option( $this->shared->get( 'slug' ) . '_fixed_flags_position', "bottom" );
		add_option( $this->shared->get( 'slug' ) . '_fixed_flags_size', "64" );
		add_option( $this->shared->get( 'slug' ) . '_fixed_flags_margin', "10" );
		add_option( $this->shared->get( 'slug' ) . '_fixed_flags_container_color', "#000000" );
		add_option( $this->shared->get( 'slug' ) . '_fixed_flags_container_opacity', "0.1" );
		add_option( $this->shared->get( 'slug' ) . '_fixed_flags_container_horizontal_padding', "12" );
		add_option( $this->shared->get( 'slug' ) . '_fixed_flags_container_vertical_padding', "2" );
		add_option( $this->shared->get( 'slug' ) . '_fixed_flags_container_border_radius', "4" );

		//Create one language option for each language available in the $languages array
		foreach ( $this->shared->languages as $iso_code => $name ) {
			add_option( $this->shared->get( 'slug' ) . '_language_' . $iso_code, '' );
		}

		//country list (ISO 3166-1 alpha-2)
		$locale = array(
			'ad' => 'Andorra',
			'ae' => 'United Arab Emirates',
			'af' => 'Afghanistan',
			'ag' => 'Antigua and Barbuda',
			'ai' => 'Anguilla',
			'al' => 'Albania',
			'am' => 'Armenia',
			'ao' => 'Angola',
			'aq' => 'Antartica',
			'ar' => 'Argentina',
			'as' => 'American Samoa',
			'at' => 'Austria',
			'au' => 'Australia',
			'aw' => 'Aruba',
			'ax' => 'Åland Islands',
			'az' => 'Azerbaijan',
			'ba' => 'Bosnia and Herzegovina',
			'bb' => 'Barbados',
			'bd' => 'Bangladesh',
			'be' => 'Belgium',
			'bf' => 'Burkina Faso',
			'bg' => 'Bulgaria',
			'bh' => 'Bahrain',
			'bi' => 'Burundi',
			'bj' => 'Benin',
			'bl' => 'Saint Barthélemy',
			'bm' => 'Bermuda',
			'bn' => 'Brunei Darussalam',
			'bo' => 'Bolivia',
			'br' => 'Brazil',
			'bs' => 'Bahamas',
			'bt' => 'Bhutan',
			'bw' => 'Botswana',
			'by' => 'Belarus',
			'bz' => 'Belize',
			'ca' => 'Canada',
			'cc' => 'Cocos (Keeling) Islands',
			'cd' => 'Congo Democratic Republic',
			'cf' => 'Central African Republic',
			'cg' => 'Congo',
			'ch' => 'Switzerland',
			'ci' => 'Côte d\'Ivoire',
			'ck' => 'Cook Islands',
			'cl' => 'Chile',
			'cm' => 'Cameroon',
			'cn' => 'China',
			'co' => 'Colombia',
			'cr' => 'Costa Rica',
			'cu' => 'Cuba',
			'cv' => 'Cape Verde',
			'cw' => 'Curaçao',
			'cx' => 'Christmas Island',
			'cy' => 'Cyprus',
			'cz' => 'Czech Republic',
			'de' => 'Germany',
			'dj' => 'Djibouti',
			'dk' => 'Denmark',
			'dm' => 'Dominica',
			'do' => 'Dominican Republic',
			'dz' => 'Algeria',
			'ec' => 'Ecuador',
			'ee' => 'Estonia',
			'eg' => 'Egypt',
			'eh' => 'Western Sahara',
			'er' => 'Eritrea',
			'es' => 'Spain',
			'et' => 'Ethiopia',
			'fi' => 'Finland',
			'fj' => 'Fiji',
			'fk' => 'Falkland Islands (Malvinas)',
			'fm' => 'Micronesia Federated States of',
			'fo' => 'Faroe Islands',
			'fr' => 'France',
			'ga' => 'Gabon',
			'gb' => 'United Kingdom',
			'gd' => 'Grenada',
			'ge' => 'Georgia',
			'gg' => 'Guernsey',
			'gh' => 'Ghana',
			'gi' => 'Gibraltar',
			'gl' => 'Greenland',
			'gm' => 'Gambia',
			'gn' => 'Guinea',
			'gq' => 'Equatorial Guinea',
			'gr' => 'Greece',
			'gs' => 'South Georgia and the South Sandwich Islands',
			'gt' => 'Guatemala',
			'gu' => 'Guam',
			'gw' => 'Guinea-Bissau',
			'gy' => 'Guyana',
			'hk' => 'Hong Kong',
			'hn' => 'Honduras',
			'hr' => 'Croatia',
			'ht' => 'Haiti',
			'hu' => 'Hungary',
			'id' => 'Indonesia',
			'ie' => 'Ireland',
			'il' => 'Israel',
			'im' => 'Isle of Man',
			'in' => 'India',
			'iq' => 'Iraq',
			'ir' => 'Iran, Islamic Republic of',
			'is' => 'Iceland',
			'it' => 'Italy',
			'je' => 'Jersey',
			'jm' => 'Jamaica',
			'jo' => 'Jordan',
			'jp' => 'Japan',
			'ke' => 'Kenya',
			'kg' => 'Kyrgyzstan',
			'kh' => 'Cambodia',
			'ki' => 'Kiribati',
			'km' => 'Comoros',
			'kn' => 'Saint Kitts and Nevis',
			'kp' => 'Korea, Democratic People\'s Republic of',
			'kr' => 'Korea, Republic of',
			'kw' => 'Kuwait',
			'ky' => 'Cayman Islands',
			'kz' => 'Kazakhstan',
			'la' => 'Lao People\'s Democratic Republic',
			'la' => 'Lebanon',
			'lc' => 'Saint Lucia',
			'li' => 'Liechtenstein',
			'lk' => 'Sri Lanka',
			'lr' => 'Liberia',
			'ls' => 'Lesotho',
			'lt' => 'Lithuania',
			'lu' => 'Luxembourg',
			'lv' => 'Latvia',
			'ly' => 'Libya',
			'ma' => 'Morocco',
			'mc' => 'Monaco',
			'md' => 'Moldova, Republic of',
			'me' => 'Montenegro',
			'mf' => 'Saint Martin (French part)',
			'mg' => 'Madagascar',
			'mh' => 'Marshall Islands',
			'mk' => 'Macedonia, the former Yugoslav Republic of',
			'ml' => 'Mali',
			'mm' => 'Myanmar',
			'mn' => 'Mongolia',
			'mo' => 'Macao',
			'mp' => 'Northern Mariana Islands',
			'mq' => 'Martinique',
			'mr' => 'Mauritania',
			'ms' => 'Montserrat',
			'mt' => 'Malta',
			'mu' => 'Mauritius',
			'mv' => 'Maldives',
			'mw' => 'Malawi',
			'mx' => 'Mexico',
			'my' => 'Malaysia',
			'mz' => 'Mozambique',
			'na' => 'Namibia',
			'nc' => 'New Caledonia',
			'ne' => 'Niger',
			'nf' => 'Norfolk Island',
			'ng' => 'Nigeria',
			'ni' => 'Nicaragua',
			'nl' => 'Netherlands',
			'no' => 'Norway',
			'np' => 'Nepal',
			'nr' => 'Nauru',
			'nu' => 'Niue',
			'nz' => 'New Zealand',
			'om' => 'Oman',
			'pa' => 'Panama',
			'pe' => 'Peru',
			'pf' => 'French Polynesia',
			'pg' => 'Papua New Guinea',
			'ph' => 'Philippines',
			'pk' => 'Pakistan',
			'pl' => 'Poland',
			'pn' => 'Pitcairn',
			'pr' => 'Puerto Rico',
			'ps' => 'Palestine, State of',
			'pt' => 'Portugal',
			'pw' => 'Palau',
			'py' => 'Paraguay',
			'qa' => 'Qatar',
			'ro' => 'Romania',
			'rs' => 'Serbia',
			'ru' => 'Russian Federation',
			'rw' => 'Rwanda',
			'sa' => 'Saudi Arabia',
			'sb' => 'Solomon Islands',
			'sc' => 'Seychelles',
			'sd' => 'Sudan',
			'se' => 'Sweden',
			'sg' => 'Singapore',
			'sh' => 'Saint Helena, Ascension and Tristan da Cunha',
			'si' => 'Slovenia',
			'sk' => 'Slovakia',
			'sl' => 'Sierra Leone',
			'sm' => 'San Marino',
			'sn' => 'Senegal',
			'so' => 'Somalia',
			'sr' => 'Suriname',
			'ss' => 'South Sudan',
			'st' => 'Sao Tome and Principe',
			'sv' => 'El Salvador',
			'sy' => 'Syrian Arab Republic',
			'sz' => 'Swaziland',
			'tc' => 'Turks and Caicos Islands',
			'td' => 'Chad',
			'tf' => 'French Southern Territories',
			'tg' => 'Togo',
			'th' => 'Thailand',
			'tj' => 'Tajikistan',
			'tk' => 'Tokelau',
			'tl' => 'Timor-Leste',
			'tm' => 'Turkmenistan',
			'tn' => 'Tunisia',
			'to' => 'Tonga',
			'tr' => 'Turkey',
			'tt' => 'Trinidad and Tobago',
			'tv' => 'Tuvalu',
			'tw' => 'Taiwan, Province of China',
			'tz' => 'Tanzania, United Republic of',
			'ua' => 'Ukraine',
			'ug' => 'Uganda',
			'us' => 'United States',
			'uy' => 'Uruguay',
			'uz' => 'Uzbekistan',
			'va' => 'Holy See (Vatican City State)',
			'vc' => 'Saint Vincent and the Grenadines',
			've' => 'Venezuela, Bolivarian Republic of',
			'vg' => 'Virgin Islands, British',
			'vi' => 'Virgin Islands, U.S.',
			'vn' => 'Viet Nam',
			'vu' => 'Vanuatu',
			'wf' => 'Wallis and Futuna',
			'ws' => 'Samoa',
			'ye' => 'Yemen',
			'yt' => 'Mayotte',
			'za' => 'South Africa',
			'zm' => 'Zambia',
			'zw' => 'Zimbabwe'
		);

		add_option( $this->shared->get( 'slug' ) . '_locale', $locale );

	}

	/*
	 * plugin delete
	 */
	static public function un_delete() {

		/*
		 * delete options and tables for all the sites in the network
		 */
		if ( function_exists( 'is_multisite' ) and is_multisite() ) {

			//get the current blog id
			global $wpdb;
			$current_blog = $wpdb->blogid;

			//create an array with all the blog ids
			$blogids = $wpdb->get_col( "SELECT blog_id FROM $wpdb->blogs" );

			//iterate through all the blogs
			foreach ( $blogids as $blog_id ) {

				//swith to the iterated blog
				switch_to_blog( $blog_id );

				//create options for the iterated blog
				Dahf_Admin::un_delete_options();

			}

			//switch to the current blog
			switch_to_blog( $current_blog );

		} else {

			/*
			 * if this is not a multisite installation delete options only for the current blog
			 */
			Dahf_Admin::un_delete_options();

		}

	}

	/*
	 * delete plugin options
	 */
	static public function un_delete_options() {

		//assign an instance of Dahf_Shared
		$shared = Dahf_Shared::get_instance();

		//General
		delete_option( $shared->get( 'slug' ) . '_flags_source' );
		delete_option( $shared->get( 'slug' ) . '_flags_type' );
		delete_option( $shared->get( 'slug' ) . '_flags_initial_opacity' );
		delete_option( $shared->get( 'slug' ) . '_language_alt_attribute');
		delete_option( $shared->get( 'slug' ) . '_language_alt_attribute_prefix');
		delete_option( $shared->get( 'slug' ) . '_language_alt_attribute_suffix');
		delete_option( $shared->get( 'slug' ) . '_locale_alt_attribute');
		delete_option( $shared->get( 'slug' ) . '_locale_alt_attribute_prefix');
		delete_option( $shared->get( 'slug' ) . '_locale_alt_attribute_suffix');

		//Fixed
		delete_option( $shared->get( 'slug' ) . '_fixed_flags' );
		delete_option( $shared->get( 'slug' ) . '_fixed_flags_size' );
		delete_option( $shared->get( 'slug' ) . '_fixed_flags_position' );
		delete_option( $shared->get( 'slug' ) . '_fixed_flags_margin' );
		delete_option( $shared->get( 'slug' ) . '_fixed_flags_mobile' );
		delete_option( $shared->get( 'slug' ) . '_fixed_flags_breakpoint' );
		delete_option( $shared->get( 'slug' ) . '_fixed_flags_container_color' );
		delete_option( $shared->get( 'slug' ) . '_fixed_flags_container_opacity' );
		delete_option( $shared->get( 'slug' ) . '_fixed_flags_container_horizontal_padding' );
		delete_option( $shared->get( 'slug' ) . '_fixed_flags_container_vertical_padding' );
		delete_option( $shared->get( 'slug' ) . '_fixed_flags_container_border_radius' );

		//Languages
		delete_option( $shared->get( 'slug' ) . '_language' );
		delete_option( $shared->get( 'slug' ) . '_locale' );

        //Delete the language options based on the $shared->languages array
        foreach ( $shared->languages as $iso_code => $name ) {
            delete_option( $shared->get( 'slug' ) . '_language_' . $iso_code );
        }

	}

	/*
	 * register the admin menu
	 */
	public function me_add_admin_menu() {

		add_menu_page(
			'HF',
			'Hreflang Flags',
			'manage_options',
			$this->shared->get( 'slug' ) . '-options',
			array( $this, 'me_display_menu_options' ),
			'dashicons-flag'
		);

		$this->screen_id_options = add_submenu_page(
			$this->shared->get( 'slug' ) . '-options',
			'HF - Options',
			'Options',
			'manage_options',
			$this->shared->get( 'slug' ) . '-options',
			array( $this, 'me_display_menu_options' )
		);

	}

	/*
	 * includes the options view
	 */
	public function me_display_menu_options() {
		include_once( 'view/options.php' );
	}

	/*
	 * register options
	 */
	public function op_register_options() {

		//Section General ----------------------------------------------------------------------------------------------
		add_settings_section(
			'dahf_general_settings_section',
			null,
			null,
			'dahf_general_options'
		);

		add_settings_field(
			'flags_source',
			esc_attr__( 'Source', 'dahf' ),
			array( $this, 'flags_source_callback' ),
			'dahf_general_options',
			'dahf_general_settings_section'
		);

		register_setting(
			'dahf_general_options',
			'dahf_flags_source',
			array( $this, 'flags_source_validation' )
		);

		add_settings_field(
			'flags_type',
			esc_attr__( 'Flags Type', 'dahf' ),
			array( $this, 'flags_type_callback' ),
			'dahf_general_options',
			'dahf_general_settings_section'
		);

		register_setting(
			'dahf_general_options',
			'dahf_flags_type',
			array( $this, 'flags_type_validation' )
		);

		add_settings_field(
			'flags_initial_opacity',
			esc_attr__( 'Flags Initial Opacity', 'dahf' ),
			array( $this, 'flags_initial_opacity_callback' ),
			'dahf_general_options',
			'dahf_general_settings_section'
		);

		register_setting(
			'dahf_general_options',
			'dahf_flags_initial_opacity',
			array( $this, 'flags_initial_opacity_validation' )
		);

		add_settings_field(
			'language_alt_attribute',
			esc_attr__( 'Language Alt Attribute', 'dahf' ),
			array( $this, 'language_alt_attribute_callback' ),
			'dahf_general_options',
			'dahf_general_settings_section'
		);

		register_setting(
			'dahf_general_options',
			'dahf_language_alt_attribute',
			array( $this, 'language_alt_attribute_validation' )
		);

		add_settings_field(
			'language_alt_attribute_prefix',
			esc_attr__( 'Language Alt Attribute Prefix', 'dahf' ),
			array( $this, 'language_alt_attribute_prefix_callback' ),
			'dahf_general_options',
			'dahf_general_settings_section'
		);

		register_setting(
			'dahf_general_options',
			'dahf_language_alt_attribute_prefix',
			array( $this, 'language_alt_attribute_prefix_validation' )
		);

		add_settings_field(
			'language_alt_attribute_suffix',
			esc_attr__( 'Language Alt Attribute Suffix', 'dahf' ),
			array( $this, 'language_alt_attribute_suffix_callback' ),
			'dahf_general_options',
			'dahf_general_settings_section'
		);

		register_setting(
			'dahf_general_options',
			'dahf_language_alt_attribute_suffix',
			array( $this, 'language_alt_attribute_suffix_validation' )
		);

		add_settings_field(
			'locale_alt_attribute',
			esc_attr__( 'Locale Alt Attribute', 'dahf' ),
			array( $this, 'locale_alt_attribute_callback' ),
			'dahf_general_options',
			'dahf_general_settings_section'
		);

		register_setting(
			'dahf_general_options',
			'dahf_locale_alt_attribute',
			array( $this, 'locale_alt_attribute_validation' )
		);

		add_settings_field(
			'locale_alt_attribute_prefix',
			esc_attr__( 'Locale Alt Attribute Prefix', 'dahf' ),
			array( $this, 'locale_alt_attribute_prefix_callback' ),
			'dahf_general_options',
			'dahf_general_settings_section'
		);

		register_setting(
			'dahf_general_options',
			'dahf_locale_alt_attribute_prefix',
			array( $this, 'locale_alt_attribute_prefix_validation' )
		);

		add_settings_field(
			'locale_alt_attribute_suffix',
			esc_attr__( 'Locale Alt Attribute Suffix', 'dahf' ),
			array( $this, 'locale_alt_attribute_suffix_callback' ),
			'dahf_general_options',
			'dahf_general_settings_section'
		);

		register_setting(
			'dahf_general_options',
			'dahf_locale_alt_attribute_suffix',
			array( $this, 'locale_alt_attribute_suffix_validation' )
		);

		add_settings_section(
			'dahf_fixed_settings_section',
			null,
			null,
			'dahf_fixed_options'
		);

		add_settings_field(
			'fixed_flags',
			esc_attr__( 'Enable', 'dahf' ),
			array( $this, 'fixed_flags_callback' ),
			'dahf_fixed_options',
			'dahf_fixed_settings_section'
		);

		register_setting(
			'dahf_fixed_options',
			'dahf_fixed_flags',
			array( $this, 'fixed_flags_validation' )
		);

		add_settings_field(
			'fixed_flags_mobile',
			esc_attr__( 'Enable Mobile', 'dahf' ),
			array( $this, 'fixed_flags_mobile_callback' ),
			'dahf_fixed_options',
			'dahf_fixed_settings_section'
		);

		register_setting(
			'dahf_fixed_options',
			'dahf_fixed_flags_mobile',
			array( $this, 'fixed_flags_mobile_validation' )
		);

		add_settings_field(
			'fixed_flags_breakpoint',
			esc_attr__( 'Breakpoint', 'dahf' ),
			array( $this, 'fixed_flags_breakpoint_callback' ),
			'dahf_fixed_options',
			'dahf_fixed_settings_section'
		);

		register_setting(
			'dahf_fixed_options',
			'dahf_fixed_flags_breakpoint',
			array( $this, 'fixed_flags_breakpoint_validation' )
		);

		add_settings_field(
			'fixed_flags_position',
			esc_attr__( 'Position', 'dahf' ),
			array( $this, 'fixed_flags_position_callback' ),
			'dahf_fixed_options',
			'dahf_fixed_settings_section'
		);

		register_setting(
			'dahf_fixed_options',
			'dahf_fixed_flags_position',
			array( $this, 'fixed_flags_position_validation' )
		);

		add_settings_field(
			'fixed_flags_size',
			esc_attr__( 'Flags Size', 'dahf' ),
			array( $this, 'fixed_flags_size_callback' ),
			'dahf_fixed_options',
			'dahf_fixed_settings_section'
		);

		register_setting(
			'dahf_fixed_options',
			'dahf_fixed_flags_size',
			array( $this, 'fixed_flags_size_validation' )
		);

		add_settings_field(
			'fixed_flags_margin',
			esc_attr__( 'Margin', 'dahf' ),
			array( $this, 'fixed_flags_margin_callback' ),
			'dahf_fixed_options',
			'dahf_fixed_settings_section'
		);

		register_setting(
			'dahf_fixed_options',
			'dahf_fixed_flags_margin',
			array( $this, 'fixed_flags_margin_validation' )
		);

		add_settings_field(
			'fixed_flags_container_color',
			esc_attr__( 'Container Color', 'dahf' ),
			array( $this, 'fixed_flags_container_color_callback' ),
			'dahf_fixed_options',
			'dahf_fixed_settings_section'
		);

		register_setting(
			'dahf_fixed_options',
			'dahf_fixed_flags_container_color',
			array( $this, 'fixed_flags_container_color_validation' )
		);

		add_settings_field(
			'fixed_flags_container_opacity',
			esc_attr__( 'Container Opacity', 'dahf' ),
			array( $this, 'fixed_flags_container_opacity_callback' ),
			'dahf_fixed_options',
			'dahf_fixed_settings_section'
		);

		register_setting(
			'dahf_fixed_options',
			'dahf_fixed_flags_container_opacity',
			array( $this, 'fixed_flags_container_opacity_validation' )
		);

		add_settings_field(
			'fixed_flags_container_horizontal_padding',
			esc_attr__( 'Container Horizontal Padding', 'dahf' ),
			array( $this, 'fixed_flags_container_horizontal_padding_callback' ),
			'dahf_fixed_options',
			'dahf_fixed_settings_section'
		);

		register_setting(
			'dahf_fixed_options',
			'dahf_fixed_flags_container_horizontal_padding',
			array( $this, 'fixed_flags_container_horizontal_padding_validation' )
		);

		add_settings_field(
			'fixed_flags_container_vertical_padding',
			esc_attr__( 'Container Vertical Padding', 'dahf' ),
			array( $this, 'fixed_flags_container_vertical_padding_callback' ),
			'dahf_fixed_options',
			'dahf_fixed_settings_section'
		);

		register_setting(
			'dahf_fixed_options',
			'dahf_fixed_flags_container_vertical_padding',
			array( $this, 'fixed_flags_container_vertical_padding_validation' )
		);

		add_settings_field(
			'fixed_flags_container_border_radius',
			esc_attr__( 'Container Border Radius', 'dahf' ),
			array( $this, 'fixed_flags_container_border_radius_callback' ),
			'dahf_fixed_options',
			'dahf_fixed_settings_section'
		);

		register_setting(
			'dahf_fixed_options',
			'dahf_fixed_flags_container_border_radius',
			array( $this, 'fixed_flags_container_border_radius_validation' )
		);

		//Section Languages --------------------------------------------------------------------------------------------
		add_settings_section(
			'dahf_languages_settings_section',
			null,
			null,
			'dahf_languages_options'
		);

		$languages = get_option( $this->shared->get( 'slug' ) . '_languages' );

		foreach ( $this->shared->languages as $iso_code => $name ) {

			add_settings_field(
				'language_' . $iso_code,
				esc_attr( $name ),
				array( $this, $iso_code . '_language_callback' ),
				'dahf_languages_options',
				'dahf_languages_settings_section'
			);

			register_setting(
				'dahf_languages_options',
				'dahf_language_' . $iso_code
			);

		}

	}

	//General options callbacks and validations ------------------------------------------------------------------------
	public function flags_source_callback( $args ) {

		$html = '<select id="dahf-flags-source" name="dahf_flags_source">';
		$html .= '<option ' . selected( get_option( "dahf_flags_source" ), 'language-locale',
				false ) . ' value="language-locale">' . esc_attr__( 'Language and Locale', 'dahf' ) . '</option>';
		$html .= '<option ' . selected( get_option( "dahf_flags_source" ), 'language',
				false ) . ' value="language">' . esc_attr__( 'Language', 'dahf' ) . '</option>';
		$html .= '<option ' . selected( get_option( "dahf_flags_source" ), 'language-priority',
				false ) . ' value="language-priority">' . esc_attr__( 'Language (Priority)', 'dahf' ) . '</option>';
		$html .= '<option ' . selected( get_option( "dahf_flags_source" ), 'locale',
				false ) . ' value="locale">' . esc_attr__( 'Locale', 'dahf' ) . '</option>';
		$html .= '<option ' . selected( get_option( "dahf_flags_source" ), 'locale-priority',
				false ) . ' value="locale-priority">' . esc_attr__( 'Locale (Priority)', 'dahf' ) . '</option>';
		$html .= '</select>';
		$html .= '<div class="help-icon" title="' . esc_attr__( 'This option determines how the information retrieved from your implementation of hreflang will be used to generate the flags.',
				'dahf' ) . '"></div>';

		echo $html;

	}

	public function flags_source_validation( $input ) {

		return $input;

	}

	public function flags_type_callback( $args ) {

		$html = '<select id="dahf-flags-type" name="dahf_flags_type">';
		$html .= '<option ' . selected( get_option( "dahf_flags_type" ), 'flat',
				false ) . ' value="flat">' . esc_attr__( 'Flat', 'dahf' ) . '</option>';
		$html .= '<option ' . selected( get_option( "dahf_flags_type" ), 'shiny',
				false ) . ' value="shiny">' . esc_attr__( 'Shiny', 'dahf' ) . '</option>';
		$html .= '</select>';
		$html .= '<div class="help-icon" title="' . esc_attr__( 'This option determines the type of flags displayed in the front-end.',
				'dahf' ) . '"></div>';

		echo $html;

	}

	public function flags_type_validation( $input ) {

		return $input;

	}

	public function flags_initial_opacity_callback( $args ) {

		$html = '<input maxlength="4" autocomplete="off" type="text" id="dahf-flags-initial_opacity" name="dahf_flags_initial_opacity" class="regular-text" value="' . esc_attr( get_option( "dahf_flags_initial_opacity" ) ) . '" />';
		$html .= '<div class="help-icon" title="' . esc_attr__( 'This option determines the initial opacity of the flags.',
				'dahf' ) . '"></div>';

		echo $html;

	}

	public function flags_initial_opacity_validation( $input ) {

		if ( ! preg_match( $this->shared->opacity_regex, $input ) ) {
			add_settings_error( 'dahf_flags_initial_opacity', 'dahf_flags_initial_opacity',
				esc_attr__( 'Please enter a valid value in the "Flags Initial Opacity" option.', 'dahf' ) );
			$output = get_option( 'dahf_flags_initial_opacity' );
		} else {
			$output = $input;
		}

		return trim( $output );

	}

	public function language_alt_attribute_callback( $args ) {

		$html = '<select id="dahf-language-alt-attribute" name="dahf_language_alt_attribute">';
		$html .= '<option ' . selected( intval( get_option( "dahf_language_alt_attribute" ), 10 ), 0,
				false ) . ' value="0">' . esc_attr__( 'No', 'dahf' ) . '</option>';
		$html .= '<option ' . selected( intval( get_option( "dahf_language_alt_attribute" ), 10 ), 1,
				false ) . ' value="1">' . esc_attr__( 'Yes', 'dahf' ) . '</option>';
		$html .= '</select>';
		$html .= '<div class="help-icon" title="' . esc_attr__( 'This option determines whether to generate or not the "alt" attribute of the image associated with the language.',
				'dahf' ) . '"></div>';

		echo $html;

	}

	public function language_alt_attribute_validation( $input ) {

		return $input;

	}

	public function language_alt_attribute_prefix_callback( $args ) {

		$html = '<input maxlength="255" autocomplete="off" type="text" id="dahf-language-alt-attribute-prefix" name="dahf_language_alt_attribute_prefix" class="regular-text" value="' . esc_attr( get_option( "dahf_language_alt_attribute_prefix" ) ) . '" />';
		$html .= '<div class="help-icon" title="' . esc_attr__( 'This option determines the prefix used in the value of the "alt" attribute of the image associated with the language.',
				'dahf' ) . '"></div>';

		echo $html;

	}

	public function language_alt_attribute_prefix_validation( $input ) {

		if ( strlen($input > 255) ) {
			add_settings_error( 'dahf_language_alt_attribute_prefix', 'dahf_language_alt_attribute_prefix',
				esc_attr__( 'Please enter a valid value in the "Language Alt Attribute Prefix" option.', 'dahf' ) );
			$output = get_option( 'dahf_language_alt_attribute_prefix' );
		} else {
			$output = $input;
		}

		return trim( $output );

	}

	public function language_alt_attribute_suffix_callback( $args ) {

		$html = '<input maxlength="255" autocomplete="off" type="text" id="dahf-language-alt-attribute-suffix" name="dahf_language_alt_attribute_suffix" class="regular-text" value="' . esc_attr( get_option( "dahf_language_alt_attribute_suffix" ) ) . '" />';
		$html .= '<div class="help-icon" title="' . esc_attr__( 'This option determines the suffix used in the value of the "alt" attribute of the image associated with the language.',
				'dahf' ) . '"></div>';

		echo $html;

	}

	public function language_alt_attribute_suffix_validation( $input ) {

		if ( strlen($input > 255) ) {
			add_settings_error( 'dahf_language_alt_attribute_suffix', 'dahf_language_alt_attribute_suffix',
				esc_attr__( 'Please enter a valid value in the "Language Alt Attribute Suffix" option.', 'dahf' ) );
			$output = get_option( 'dahf_language_alt_attribute_suffix' );
		} else {
			$output = $input;
		}

		return trim( $output );

	}

	public function locale_alt_attribute_callback( $args ) {

		$html = '<select id="dahf-locale-alt-attribute" name="dahf_locale_alt_attribute">';
		$html .= '<option ' . selected( intval( get_option( "dahf_locale_alt_attribute" ), 10 ), 0,
				false ) . ' value="0">' . esc_attr__( 'No', 'dahf' ) . '</option>';
		$html .= '<option ' . selected( intval( get_option( "dahf_locale_alt_attribute" ), 10 ), 1,
				false ) . ' value="1">' . esc_attr__( 'Yes', 'dahf' ) . '</option>';
		$html .= '</select>';
		$html .= '<div class="help-icon" title="' . esc_attr__( 'This option determines whether to generate or not the "alt" attribute of the image associated with the locale.',
				'dahf' ) . '"></div>';

		echo $html;

	}

	public function locale_alt_attribute_validation( $input ) {

		return $input;

	}

	public function locale_alt_attribute_prefix_callback( $args ) {

		$html = '<input maxlength="255" autocomplete="off" type="text" id="dahf-locale-alt-attribute-prefix" name="dahf_locale_alt_attribute_prefix" class="regular-text" value="' . esc_attr( get_option( "dahf_locale_alt_attribute_prefix" ) ) . '" />';
		$html .= '<div class="help-icon" title="' . esc_attr__( 'This option determines the prefix used in the value of the "alt" attribute of the image associated with the locale.',
				'dahf' ) . '"></div>';

		echo $html;

	}

	public function locale_alt_attribute_prefix_validation( $input ) {

		if ( strlen($input > 255) ) {
			add_settings_error( 'dahf_locale_alt_attribute_prefix', 'dahf_locale_alt_attribute_prefix',
				esc_attr__( 'Please enter a valid value in the "Locale Alt Attribute Prefix" option.', 'dahf' ) );
			$output = get_option( 'dahf_locale_alt_attribute_prefix' );
		} else {
			$output = $input;
		}

		return trim( $output );

	}

	public function locale_alt_attribute_suffix_callback( $args ) {

		$html = '<input maxlength="255" autocomplete="off" type="text" id="dahf-locale-alt-attribute-suffix" name="dahf_locale_alt_attribute_suffix" class="regular-text" value="' . esc_attr( get_option( "dahf_locale_alt_attribute_suffix" ) ) . '" />';
		$html .= '<div class="help-icon" title="' . esc_attr__( 'This option determines the suffix used in the value of the "alt" attribute of the image associated with the locale.',
				'dahf' ) . '"></div>';

		echo $html;

	}

	public function locale_alt_attribute_suffix_validation( $input ) {

		if ( strlen($input > 255) ) {
			add_settings_error( 'dahf_locale_alt_attribute_suffix', 'dahf_locale_alt_attribute_suffix',
				esc_attr__( 'Please enter a valid value in the "Locale Alt Attribute Suffix" option.', 'dahf' ) );
			$output = get_option( 'dahf_locale_alt_attribute_suffix' );
		} else {
			$output = $input;
		}

		return trim( $output );

	}

	public function fixed_flags_callback( $args ) {

		$html = '<select id="dahf-fixed-flags" name="dahf_fixed_flags">';
		$html .= '<option ' . selected( intval( get_option( "dahf_fixed_flags" ), 10 ), 0,
				false ) . ' value="0">' . esc_attr__( 'No', 'dahf' ) . '</option>';
		$html .= '<option ' . selected( intval( get_option( "dahf_fixed_flags" ), 10 ), 1,
				false ) . ' value="1">' . esc_attr__( 'Yes', 'dahf' ) . '</option>';
		$html .= '</select>';
		$html .= '<div class="help-icon" title="' . esc_attr__( 'This option determines whether to enable or not the fixed flags.',
				'dahf' ) . '"></div>';

		echo $html;

	}

	public function fixed_flags_validation( $input ) {

		return $input;

	}

	public function fixed_flags_size_callback( $args ) {

		$html = '<select id="dahf-fixed-flags-size" name="dahf_fixed_flags_size">';
		$html .= '<option ' . selected( get_option( "dahf_fixed_flags_size" ), '16',
				false ) . ' value="16">16</option>';
		$html .= '<option ' . selected( get_option( "dahf_fixed_flags_size" ), '24',
				false ) . ' value="24">24</option>';
		$html .= '<option ' . selected( get_option( "dahf_fixed_flags_size" ), '32',
				false ) . ' value="32">32</option>';
		$html .= '<option ' . selected( get_option( "dahf_fixed_flags_size" ), '48',
				false ) . ' value="48">48</option>';
		$html .= '<option ' . selected( get_option( "dahf_fixed_flags_size" ), '64',
				false ) . ' value="64">64</option>';
		$html .= '</select>';
		$html .= '<div class="help-icon" title="' . esc_attr__( 'This option determines the size of the flags used in the fixed flags.',
				'dahf' ) . '"></div>';

		echo $html;

	}

	public function fixed_flags_size_validation( $input ) {

		return $input;

	}

	public function fixed_flags_position_callback( $args ) {

		$html = '<select id="dahf-fixed-flags-position" name="dahf_fixed_flags_position">';
		$html .= '<option ' . selected( get_option( "dahf_fixed_flags_position" ), 'top',
				false ) . ' value="top">' . esc_attr__( 'Top', 'dahf' ) . '</option>';
		$html .= '<option ' . selected( get_option( "dahf_fixed_flags_position" ), 'right',
				false ) . ' value="right">' . esc_attr__( 'Right', 'dahf' ) . '</option>';
		$html .= '<option ' . selected( get_option( "dahf_fixed_flags_position" ), 'bottom',
				false ) . ' value="bottom">' . esc_attr__( 'Bottom', 'dahf' ) . '</option>';
		$html .= '<option ' . selected( get_option( "dahf_fixed_flags_position" ), 'left',
				false ) . ' value="left">' . esc_attr__( 'Left', 'dahf' ) . '</option>';
		$html .= '</select>';
		$html .= '<div class="help-icon" title="' . esc_attr__( 'This option determines the position of the fixed flags.',
				'dahf' ) . '"></div>';

		echo $html;

	}

	public function fixed_flags_position_validation( $input ) {

		return $input;

	}

	public function fixed_flags_margin_callback( $args ) {

		$html = '<input maxlength="4" autocomplete="off" type="text" id="dahf-fixed-flags-margin" name="dahf_fixed_flags_margin" class="regular-text" value="' . esc_attr( get_option( "dahf_fixed_flags_margin" ) ) . '" />';
		$html .= '<div class="help-icon" title="' . esc_attr__( 'This option determines the space between the flags.',
				'dahf' ) . '"></div>';

		echo $html;

	}

	public function fixed_flags_margin_validation( $input ) {

		if ( ! preg_match( $this->shared->digits_regex, $input ) ) {
			add_settings_error( 'dahf_fixed_flags_margin', 'dahf_fixed_flags_margin',
				esc_attr__( 'Please enter a valid value in the "Margin" option.', 'dahf' ) );
			$output = get_option( 'dahf_fixed_flags_margin' );
		} else {
			$output = $input;
		}

		return trim( $output );

	}

	public function fixed_flags_mobile_callback( $args ) {

		$html = '<select id="dahf-fixed-flags-mobile" name="dahf_fixed_flags_mobile">';
		$html .= '<option ' . selected( intval( get_option( "dahf_fixed_flags_mobile" ), 10 ), 0,
				false ) . ' value="0">' . esc_attr__( 'No', 'dahf' ) . '</option>';
		$html .= '<option ' . selected( intval( get_option( "dahf_fixed_flags_mobile" ), 10 ), 1,
				false ) . ' value="1">' . esc_attr__( 'Yes', 'dahf' ) . '</option>';
		$html .= '</select>';
		$html .= '<div class="help-icon" title="' . esc_attr__( 'Select "Yes" to enable the fixed flags with mobile devices or "No" to disable the fixed flags with mobile devices. The distinction between a desktop device and a mobile device is performed by analyzing the user agent string of the browser.',
				'dahf' ) . '"></div>';

		echo $html;

	}

	public function fixed_flags_mobile_validation( $input ) {

		return $input;

	}

	public function fixed_flags_breakpoint_callback( $args ) {

		$html = '<input maxlength="4" autocomplete="off" type="text" id="dahf-fixed-flags-breakpoint" name="dahf_fixed_flags_breakpoint" class="regular-text" value="' . esc_attr( get_option( "dahf_fixed_flags_breakpoint" ) ) . '" />';
		$html .= '<div class="help-icon" title="' . esc_attr__( 'This option allows your to automatically hide the fixed flags when the browser viewport width goes below the specified value in pixels.',
				'dahf' ) . '"></div>';

		echo $html;

	}

	public function fixed_flags_breakpoint_validation( $input ) {

		if ( ! preg_match( $this->shared->digits_regex, $input ) ) {
			add_settings_error( 'dahf_fixed_flags_breakpoint', 'dahf_fixed_flags_breakpoint',
				esc_attr__( 'Please enter a valid value in the "Breakpoint" option.', 'dahf' ) );
			$output = get_option( 'dahf_fixed_flags_breakpoint' );
		} else {
			$output = $input;
		}

		return trim( $output );

	}

	public function fixed_flags_container_color_callback( $args ) {

		$html = '<input autocomplete="off" type="text" id="dahf-fixed-flags-container-color" name="dahf_fixed_flags_container_color" class="regular-text wp-color-picker" value="' . esc_attr( get_option( "dahf_fixed_flags_container_color" ) ) . '" />';
		$html .= '<div class="help-icon" title="' . esc_attr__( 'The color of the fixed flags container.',
				'dahf' ) . '"></div>';

		echo $html;

	}

	public function fixed_flags_container_color_validation( $input ) {

		if ( ! preg_match( $this->shared->hex_rgb_regex, $input ) ) {
			add_settings_error( 'dahf_fixed_flags_container_color', 'dahf_fixed_flags_container_color',
				esc_attr__( 'Please enter a valid value in the "Container Color" option.', 'dahf' ) );
			$output = get_option( 'dahf_fixed_flags_container_color' );
		} else {
			$output = $input;
		}

		return trim( $output );

	}

	public function fixed_flags_container_opacity_callback( $args ) {

		$html = '<input maxlength="4" autocomplete="off" type="text" id="dahf-fixed-flags-opacity" name="dahf_fixed_flags_container_opacity" class="regular-text" value="' . esc_attr( get_option( "dahf_fixed_flags_container_opacity" ) ) . '" />';
		$html .= '<div class="help-icon" title="' . esc_attr__( 'Enter a value included between 0 and 1 to set the opacity of the fixed flags container.',
				'dahf' ) . '"></div>';

		echo $html;

	}

	public function fixed_flags_container_opacity_validation( $input ) {

		if ( ! preg_match( $this->shared->opacity_regex, $input ) ) {
			add_settings_error( 'dahf_fixed_flags_container_opacity', 'dahf_fixed_flags_container_opacity',
				esc_attr__( 'Please enter a valid value in the "Container Opacity" option.', 'dahf' ) );
			$output = get_option( 'dahf_fixed_flags_container_opacity' );
		} else {
			$output = $input;
		}

		return trim( $output );

	}

	public function fixed_flags_container_horizontal_padding_callback( $args ) {

		$html = '<input maxlength="4" autocomplete="off" type="text" id="dahf-fixed-flags-container-horizontal-padding" name="dahf_fixed_flags_container_horizontal_padding" class="regular-text" value="' . esc_attr( get_option( "dahf_fixed_flags_container_horizontal_padding" ) ) . '" />';
		$html .= '<div class="help-icon" title="' . esc_attr__( 'The horizontal padding of the fixed flags container.',
				'dahf' ) . '"></div>';

		echo $html;

	}

	public function fixed_flags_container_horizontal_padding_validation( $input ) {

		if ( ! preg_match( $this->shared->digits_regex, $input ) ) {
			add_settings_error( 'dahf_fixed_flags_container_horizontal_padding',
				'dahf_fixed_flags_container_horizontal_padding',
				esc_attr__( 'Please enter a valid value in the "Container Horizontal Padding" option.', 'dahf' ) );
			$output = get_option( 'dahf_fixed_flags_container_horizontal_padding' );
		} else {
			$output = $input;
		}

		return trim( $output );

	}

	public function fixed_flags_container_vertical_padding_callback( $args ) {

		$html = '<input maxlength="4" autocomplete="off" type="text" id="dahf-fixed-flags-container-vertical-padding" name="dahf_fixed_flags_container_vertical_padding" class="regular-text" value="' . esc_attr( get_option( "dahf_fixed_flags_container_vertical_padding" ) ) . '" />';
		$html .= '<div class="help-icon" title="' . esc_attr__( 'The vertical padding of the fixed flags container.',
				'dahf' ) . '"></div>';

		echo $html;

	}

	public function fixed_flags_container_vertical_padding_validation( $input ) {

		if ( ! preg_match( $this->shared->digits_regex, $input ) ) {
			add_settings_error( 'dahf_fixed_flags_container_vertical_padding',
				'dahf_fixed_flags_container_vertical_padding',
				esc_attr__( 'Please enter a valid value in the "Container Vertical Padding" option.', 'dahf' ) );
			$output = get_option( 'dahf_fixed_flags_container_vertical_padding' );
		} else {
			$output = $input;
		}

		return trim( $output );

	}

	public function fixed_flags_container_border_radius_callback( $args ) {

		$html = '<input maxlength="4" autocomplete="off" type="text" id="dahf-fixed-flags-container_border_radius" name="dahf_fixed_flags_container_border_radius" class="regular-text" value="' . esc_attr( get_option( "dahf_fixed_flags_container_border_radius" ) ) . '" />';
		$html .= '<div class="help-icon" title="' . esc_attr__( 'The border radius of the fixed flags container.',
				'dahf' ) . '"></div>';

		echo $html;

	}

	public function fixed_flags_container_border_radius_validation( $input ) {

		if ( ! preg_match( $this->shared->digits_regex, $input ) ) {
			add_settings_error( 'dahf_fixed_flags_container_border_radius', 'dahf_fixed_flags_container_border_radius',
				esc_attr__( 'Please enter a valid value in the "Container Border Radius" option.', 'dahf' ) );
			$output = get_option( 'dahf_fixed_flags_container_border_radius' );
		} else {
			$output = $input;
		}

		return trim( $output );

	}

	//Languages options callbacks and validations ----------------------------------------------------------------------

    /**
     * This method is used to output the HTML of the options displayed in the "Languages" tab of the
     * "Options" menu.
     *
     * Any non-existing function is intercepted by _call, and the functions which have a name that
     * ends with "_language_callback" generates the HTML of the option.
     *
     * @param $func
     * @param $params
     */
	public function __call( $func, $params ) {

		//if this is not a language callback function return
		if ( substr( $func, - 18 ) != '_language_callback' ) {
			return;
		}

		$iso_code  = substr( $func, 0, 2 );
        $languages = $this->shared->languages;

		foreach ( $languages as $key => $value ) {

			if ( $iso_code == $key ) {

				//callback
				$html = '<select id="dahf-language-' . $iso_code . '" name="dahf_language_' . $iso_code . '">';

				$locale = get_option( $this->shared->get( 'slug' ) . '_locale' );

				$html .= '<option ' . selected( get_option( $this->shared->get( 'slug' ) . '_language_' . $key ), '',
						false ) . ' value="">' . esc_attr__( 'Not assigned', 'dahf' ) . '</option>';

				foreach ( $locale as $key2 => $value2 ) {

					$html .= '<option ' . selected( get_option( $this->shared->get( 'slug' ) . '_language_' . $key ),
							$key2, false ) . ' value="' . $key2 . '">' . esc_attr__( $value2, 'dahf' ) . '</option>';

				}

				$html .= '</select>';
				$html .= '<div class="help-icon" title="' . esc_attr__( 'This option determines which flag will be associated with the language',
						'dahf' ) . ' ' . esc_attr( $value ) . '."></div>';

				echo $html;

			}

		}

	}

	/**
	 * Add the proper filters used to register the TinyMCE plugin and the TinyMCE button
	 */
	function create_tinymce_plugin() {

		//The TinyMCE plugin is created only for the users with the 'edit_posts' capability
		if ( ! current_user_can( 'edit_posts' ) ) {
			return;
		}

		if ( get_user_option( 'rich_editing' ) == 'true' ) {

			//filter used to create the tinymce plugin
			add_filter( 'mce_external_plugins', array( $this, 'add_tinymce_plugin' ) );

			//filter used to create the tinymce button
			add_filter( 'mce_buttons', array( $this, 'register_tinymce_buttons' ) );

		}

	}

	//Add the 'hreflangflags' TinyMCE Plugin
	public function add_tinymce_plugin( $plugin_array ) {

		$plugin_array['hreflangflags'] = $this->shared->get( "url" ) . 'admin/assets/js/tinymce-hreflangflags.js';

		return $plugin_array;

	}

	/*
	 * Add the "selecthreflangflags" tinymce button
	 *
	 * @return array
	 */
	public function register_tinymce_buttons( $buttons ) {

		array_push( $buttons, "", "selecthreflangflags" );

		return $buttons;

	}

}