(function($, hreflangFlags) {

  $(document).ready(function() {

    /**
     * Get the information available in the link tags with the hreflang attribute and the related URL
     * available in the href attribute and save them in an array.
     */
    var attributeHreflang = '';
    var attributeHref = '';
    var languageIsoCode = '';
    var localeIsoCode = '';

    //This array stores the hreflang data found in the link elements with the hreflang attribute found in the page head
    var collectedHreflang = [];

    //Iterate over all the link elements with the href and rel attributes
    $('head link[hreflang][href][rel]').each(function() {

      //Get the values of the attributes
      attributeHreflang = $(this).attr('hreflang');
      attributeHref = $(this).attr('href');
      var detectedLanguage = '';
      var detectedLocale = '';

      //Save the languageIsoCode and the localeIsoCode
      if (attributeHreflang.length === 2) {

        //Set an empty string if the language-locale association is not defined
        if(typeof hreflangFlags.language[attributeHreflang] === 'undefined'){
          languageIsoCode = '';
        }else{
          languageIsoCode = hreflangFlags.language[attributeHreflang];
        }

        localeIsoCode = '';
        detectedLanguage = attributeHreflang;

      } else if (attributeHreflang.length === 5) {

        //Set an empty string if the language-locale association is not defined
        if(typeof hreflangFlags.language[attributeHreflang.substr(0, 2)] === 'undefined'){
          languageIsoCode = '';
        }else{
          languageIsoCode = hreflangFlags.language[attributeHreflang.substr(0, 2)];
        }

        localeIsoCode = attributeHreflang.substr(3, 5);
        detectedLanguage = attributeHreflang.substr(0, 2);
        detectedLocale = attributeHreflang.substr(3, 5)

      }else{
        return;
      }

      /**
       * Adds the data of this single link element to the collectedHreflang
       * array.
       *
       * Please note that:
       *
       * - The detectedLanguage and detectedLocale properties are the exact
       * hreflang codes retrieved from the page HTML.
       *
       * - The language and locale properties are the codes that will be used
       * for the actual flags. These code are based on the hreflang codes
       * retrieved from the page HTML, but are then converted based on the
       * language/locale hash table stored in the hreflangFlags.language object
       * property. Note that the hreflangFlags.language object property is
       * based on the selections performed in the "Languages" section of the
       * plugin options.
       */
      collectedHreflang.push({
        detectedLanguage: detectedLanguage,
        detectedLocale: detectedLocale,
        language: languageIsoCode.toUpperCase(),
        locale: localeIsoCode.toUpperCase(),
        url: attributeHref,
      });

    });

    /**
     * Adds the HTML and add the flags to the DOM if there are hreflang data in the collectedHreflang array.
     *
     * If there are no data in the collectedHreflang array delete from the DOM the "Hreflang Flags" Widget area, the
     * placeholders of the [hreflang-flags] shortcode and the placeholders of the hreflang_flags() PHP function.
     */
    if (collectedHreflang.length > 0) {

      /**
       * Generated the HTML used to display the flags
       */
      var colors = [];
      var html = '<div class="hreflang-flags-container">';
      var partialUrl = hreflangFlags.flagsBaseUrl + hreflangFlags.flagsType + '/' + '[flags-size-placeholder]' + '/';
      $.each(collectedHreflang, function(index, value) {

        //If the language is 'Not Assigned' use the question mark flag
        if (value.language.length === 0) {
          value.language = '_u';
        }

        /**
         * Based on the hreflangFlags.flagsSource option (which determines how the information retrieve from the implementation
         * of the hreflang should be used to generate the flags) generate the HTML used to display the flags.
         */
        switch (hreflangFlags.flagsSource) {

            //Generate a flag for the language and a flag for the locale both include in the same link element
          case 'language-locale':

            html += '<a href="' + value.url + '">';
            if (value.language.length === 2) {
              html += '<img data-src="' + partialUrl + value.language + '.png" ' + getLanguageAltAttribute(value.detectedLanguage) + '>';
            }
            if (value.locale.length === 2) {
              html += '<img data-src="' + partialUrl + value.locale + '.png" ' + getLocaleAltAttribute(value.detectedLocale) +'>';
            }
            html += '</a>';
            break;

            //Generate a flag for the language
          case 'language':

            if (value.language.length === 2) {
              html += '<a href="' + value.url + '">';
              html += '<img data-src="' + partialUrl + value.language + '.png" ' + getLanguageAltAttribute(value.detectedLanguage) +'>';
              html += '</a>';
            }
            break;

            //Generate a flag for the language if the language is available, otherwise generate a flag for the locale
          case 'language-priority':

            if (value.language.length === 2) {
              html += '<a href="' + value.url + '">';
              html += '<img data-src="' + partialUrl + value.language + '.png" ' + getLanguageAltAttribute(value.detectedLanguage) +'>';
              html += '</a>';
            } else if (value.locale.length === 2) {
              html += '<a href="' + value.url + '">';
              html += '<img data-src="' + partialUrl + value.locale + '.png" ' + getLocaleAltAttribute(value.detectedLocale) +'>';
              html += '</a>';
            }
            break;

            //Generate a flag for the locale
          case 'locale':

            if (value.locale.length === 2) {
              html += '<a href="' + value.url + '">';
              html += '<img data-src="' + partialUrl + value.locale + '.png" ' + getLocaleAltAttribute(value.detectedLocale) +'>';
              html += '</a>';
            }
            break;

            //Generate a flag for the locale if the locale is available, otherwise generate a flag for the language
          case 'locale-priority':

            if (value.locale.length === 2) {
              html += '<a href="' + value.url + '">';
              html += '<img data-src="' + partialUrl + value.locale + '.png" ' + getLocaleAltAttribute(value.detectedLocale) +'>';
              html += '</a>';
            } else if (value.language.length === 2) {
              html += '<a href="' + value.url + '">';
              html += '<img data-src="' + partialUrl + value.language + '.png" ' + getLanguageAltAttribute(value.detectedLanguage) +'>';
              html += '</a>';
            }
            break;

        }

      });
      html += '</div>';

      /**
       * If there are no flags remove the widget container (.dahf-widget) and the flags placeholder (.dahf-placefolder)
       * from the DOM. Otherwise use the HTML to generate the fixed flags and to replace the placeholders.
       */
      if(html === '<div class="hreflang-flags-container"></div>'){

        $('.dahf-widget, .dahf-placeholder').remove();

      }else{

        /**
         * Place the HTML in the fixed position if fixed is enabled and add the class for fixed and the class to assign
         * the proper position via css (top, right, bottom, left)
         */
        if(parseInt(hreflangFlags.fixedFlags, 10) == 1){

          /**
           * Browser detection with the 'isMobile' library:
           *
           * https://github.com/kaimallea/isMobile
           *
           * If the "Enable Mobile" option is set to "No" (0) verify if the device is a mobile device with the isMobile
           * object, and if the device is a mobile device the fixed flags are not generated.
           *
           */
          if (parseInt(hreflangFlags.fixedFlagsMobile, 10) == 1 || (parseInt(hreflangFlags.fixedFlagsMobile, 10) == 0 && isMobile.any === false)) {

            var htmlForFixedPosition = $.parseHTML(html);

            //Add classes
            $(htmlForFixedPosition).attr('id', 'dahf-fixed-position');
            $(htmlForFixedPosition).addClass('dahf-fixed-position');
            $(htmlForFixedPosition).addClass('dahf-fixed-position-' + hreflangFlags.fixedFlagsPosition);

            //Iterate over all the links elements
            $(htmlForFixedPosition).find('a').each(function(index) {

              /**
               * Generate the src attribute by replacing the placeholder in the data-src attribute with the value of the
               * hreflangFlags.fixedFlagsSize option
               */
              $(this).find('img').each(function() {
                $(this).attr('src', $(this).attr('data-src').replace('[flags-size-placeholder]', hreflangFlags.fixedFlagsSize));
                $(this).removeAttr('data-src');
              });

              //Set the margin
              if (index > 0) {
                if (hreflangFlags.fixedFlagsPosition === 'right' || hreflangFlags.fixedFlagsPosition === 'left') {
                  $(this).css('margin-top', hreflangFlags.fixedFlagsMargin + 'px');
                } else {
                  $(this).css('margin-left', hreflangFlags.fixedFlagsMargin + 'px');
                }
              }

            });

            //Fixed Flags Container Color
            colors = rgbHexToDec(hreflangFlags.fixedFlagsContainerColor);
            $(htmlForFixedPosition).
                css('background', 'rgba(' + colors['r'] + ', ' + colors['g'] + ', ' + colors['b'] + ', ' +
                    parseFloat(hreflangFlags.fixedFlagsContainerOpacity) + ')');

            //Fixed Flags Container Padding
            $(htmlForFixedPosition).
                css('padding', hreflangFlags.fixedFlagsContainerVerticalPadding + 'px ' +
                    hreflangFlags.fixedFlagsContainerHorizontalPadding + 'px');

            //Fixed Flags Container Border Radius
            var br = hreflangFlags.fixedFlagsContainerBorderRadius;
            switch (hreflangFlags.fixedFlagsPosition) {
              case 'top':
                $(htmlForFixedPosition).css('border-radius', '0 0 ' + br + 'px ' + br + 'px');
                break;
              case 'right':
                $(htmlForFixedPosition).css('border-radius', br + 'px 0 0 ' + br + 'px');
                break;
              case 'bottom':
                $(htmlForFixedPosition).css('border-radius', br + 'px ' + br + 'px 0 0');
                break;
              case 'left':
                $(htmlForFixedPosition).css('border-radius', '0 ' + br + 'px ' + br + 'px 0');
                break;
            }

            //Set initial Opacity
            $(htmlForFixedPosition).find('img').css('opacity', hreflangFlags.flagsInitialOpacity);

            //Add the HTML of the fixed flags to the DOM
            $('body').append(htmlForFixedPosition);

            //Set attribute
            $('#dahf-fixed-position').attr('data-flags-size', hreflangFlags.fixedFlagsSize);

            //Move the fixed flags in the appropriate position
            if (hreflangFlags.fixedFlagsPosition === 'right' || hreflangFlags.fixedFlagsPosition === 'left') {
              $('#dahf-fixed-position').
                  css('top', ($(window).height() - parseInt($('#dahf-fixed-position').css('height'), 10)) / 2);
            } else {
              $('#dahf-fixed-position').
                  css('left', ($(window).width() - parseInt($('#dahf-fixed-position').css('width'), 10)) / 2);
            }

            //Make the fixed flags visible or keep it hidden
            showHideFixedFlagsbasedOnBreakpoint();

          }

        }

        /**
         * Place the HTML in place of the placeholders generated with:
         *
         * - Shortcodes
         * - hreflang_flags() function
         * - The "Hreflang Flags" Widget
         */
        $('.dahf-placeholder').each(function() {

          var htmlForPlaceholder = $.parseHTML(html);

          //Flags Size
          var flags_size = $(this).attr('data-flags-size');
          $(htmlForPlaceholder).attr('data-flags-size', flags_size);

          //Add class
          $(htmlForPlaceholder).addClass('dahf-not-fixed-position');

          //Placeholder Orientation
          var orientation = $(this).attr('data-orientation');
          $(htmlForPlaceholder).addClass('dahf-orientation-' + orientation);

          //Iterate over all the links elements
          var margin = $(this).attr('data-margin');
          $(htmlForPlaceholder).find('a').each(function(index) {

            /**
             * Generate the src attribute by replacing the placeholder in the data-src attribute with the value of the
             * flags_size
             */
            $(this).find('img').each(function() {
              $(this).attr('src', $(this).attr('data-src').replace('[flags-size-placeholder]', flags_size));
              $(this).removeAttr('data-src');
            });

            //Set the margin
            if (index > 0) {
              if (orientation === 'horizontal') {
                $(this).css('margin-left', margin + 'px');
              } else {
                $(this).css('margin-top', margin + 'px');
              }
            }

          });

          //Placeholder Container Color
          var containerColor = $(this).attr('data-container-color');
          var containerOpacity = $(this).attr('data-container-opacity');
          colors = rgbHexToDec(containerColor);
          $(htmlForPlaceholder).
              css('background',
                  'rgba(' + colors['r'] + ', ' + colors['g'] + ', ' + colors['b'] + ', ' + parseFloat(containerOpacity) +
                  ')');

          //Placeholder Container Padding
          $(htmlForPlaceholder).
              css('padding', $(this).attr('data-container-vertical-padding') + 'px ' +
                  $(this).attr('data-container-horizontal-padding') + 'px');

          //Placeholder Container Border Radius
          $(htmlForPlaceholder).css('border-radius', parseInt($(this).attr('data-container-border-radius'), 10) + 'px');

          //Set initial Opacity
          $(htmlForPlaceholder).find('img').css('opacity', hreflangFlags.flagsInitialOpacity);

          //Add the flags HTML to the DOM
          $(this).after(htmlForPlaceholder);

        });

        //Remove all the placeholders
        $('.dahf-placeholder').remove();

        //Add event listeners
        $('.hreflang-flags-container a').mouseenter(function() {
          $(this).find('img').css('opacity', '1');
        });

        $('.hreflang-flags-container a').mouseout(function() {
          $(this).find('img').css('opacity', hreflangFlags.flagsInitialOpacity);
        });

        //Display the "Hreflang Flags" widget area
        $('.dahf-widget').show();

      }

    } else {
      $('.dahf-widget, .dahf-placeholder').remove();
    }

    /**
     *  Resize event listener.
     */
    $(window).resize(function() {

      //Do nothing if the fixed flag container doesn't exist
      if($('#dahf-fixed-position').length == false){
        return;
      }

      //Update the position of the fixed element
      if (hreflangFlags.fixedFlagsPosition === 'right' || hreflangFlags.fixedFlagsPosition === 'left') {
        $('#dahf-fixed-position').
            css('top', ($(window).height() - parseInt($('#dahf-fixed-position').css('height'), 10)) / 2);
      } else {
        $('#dahf-fixed-position').
            css('left', ($(window).width() - parseInt($('#dahf-fixed-position').css('width'), 10)) / 2);
      }

      //Show or hide the fixed flags if the browser viewport width goes below the "Breakpoint" value
      showHideFixedFlagsbasedOnBreakpoint();

    });

  });

  /*
   * Given an hexadecimal rgb color an array with the 3 components converted in decimal is returned
   *
   * @param string The hexadecimal rgb color
   * @return array An array with the 3 components of the color converted in decimal
   */
  function rgbHexToDec(hex) {

    var r, g, b;

    //remove the # character
    hex = hex.replace('#', '');

    //find the component of the color
    if (hex.length == 3) {
      r = parseInt(hex.substring(0, 1), 16);
      g = parseInt(hex.substring(1, 2), 16);
      b = parseInt(hex.substring(2, 3), 16);
    } else {
      r = parseInt(hex.substring(0, 2), 16);
      g = parseInt(hex.substring(2, 4), 16);
      b = parseInt(hex.substring(4, 6), 16);
    }

    //generate the array with the component of the color
    var color_a = new Array();
    color_a['r'] = r;
    color_a['g'] = g;
    color_a['b'] = b;

    return color_a;

  }

  /**
   * Hide the fixed flags if the browser viewport width is smaller than the "Breakpoint" value defined through
   * the plugin options or show the fixed flags if the browser viewport width is larger than the "Breakpoint" value
   * defined through the plugin options.
   */
  function showHideFixedFlagsbasedOnBreakpoint() {

    if ($(window).width() < hreflangFlags.fixedFlagsBreakpoint) {
      $('#dahf-fixed-position').css('visibility', 'hidden');
    } else {
      $('#dahf-fixed-position').css('visibility', 'visible');
    }

  }

  /**
   * Get the language name from the provided language code.
   *
   * Note that this function search in the hreflangFlags.languageCodeName
   * object property, which stores an hash table with language codes and
   * language names.
   *
   * @param languageCode
   * @returns String The locale name.
   */
  function getLanguageName(languageCode){

    for (var property in hreflangFlags.languageCodeName) {
      if (hreflangFlags.languageCodeName.hasOwnProperty(property)) {
          if(property === languageCode){
            return hreflangFlags.languageCodeName[property];
          }
      }
    }

  }

  /**
   * Get the alt attribute of the image from the provided language code.
   *
   * Note that this function search in the hreflangFlags.languageCodeName
   * object property, which stores an hash table with language codes and
   * language names.
   *
   * @param languageCode
   * @returns String The language name.
   */
  function getLanguageAltAttribute(languageCode){

    if(parseInt(hreflangFlags.languageAltAttribute, 10) === 1){
      for (var property in hreflangFlags.languageCodeName) {
        if (hreflangFlags.languageCodeName.hasOwnProperty(property)) {
          if(property === languageCode){
            var attribute = 'alt="';
            if(hreflangFlags.languageAltAttributePrefix.length > 0){
              attribute += hreflangFlags.languageAltAttributePrefix + ' ';
            }
            attribute += hreflangFlags.languageCodeName[property];
            if(hreflangFlags.languageAltAttributeSuffix.length > 0){
              attribute += ' ' + hreflangFlags.languageAltAttributeSuffix;
            }
            attribute += '"';
            return attribute;
          }
        }
      }
    }else{
      return '';
    }

  }

  /**
   * Get the alt attribute of the image from the provided locale code.
   *
   * Note that this function search in the hreflangFlags.localeCodeName
   * object property, which stores an hash table with locale codes and locale
   * names.
   *
   * @param localeCode
   * @returns String The locale name.
   */
  function getLocaleAltAttribute(localeCode){

    if(parseInt(hreflangFlags.localeAltAttribute, 10) === 1){
      for (var property in hreflangFlags.localeCodeName) {
        if (hreflangFlags.localeCodeName.hasOwnProperty(property)) {
          if(property === localeCode){
            var attribute = 'alt="';
            if(hreflangFlags.localeAltAttributePrefix.length > 0){
              attribute += hreflangFlags.localeAltAttributePrefix + ' ';
            }
            attribute += hreflangFlags.localeCodeName[property];
            if(hreflangFlags.localeAltAttributeSuffix.length > 0){
              attribute += ' ' + hreflangFlags.localeAltAttributeSuffix;
            }
            attribute += '"';
            return attribute;
          }
        }
      }
    }else{
      return '';
    }

  }

}(jQuery, hreflangFlags));