<?php

/*
 * This class should be used to work with the public side of WordPress
 */

class Dahf_Public {

	//general class properties
	protected static $instance = null;
	private $shared = null;

	private function __construct() {

		//assign an instance of the plugin info
		$this->shared = Dahf_Shared::get_instance();

		//Load public css
		add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_styles' ) );

		//Load public js
		add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_scripts' ) );

		//write in front-end head
		add_action( 'wp_head', array( $this, 'wr_public_head' ) );

		//[hreflang-flags] shortcode
		add_shortcode( 'hreflang-flags', array( $this, 'generate_placeholder' ) );

	}

	/*
	 * create an instance of this class
	 */
	public static function get_instance() {

		if ( null == self::$instance ) {
			self::$instance = new self;
		}

		return self::$instance;

	}

	public function generate_placeholder( $atts ) {

		/*
		 * Parse the shortcode only inside the full content of posts and pages is displayed. Do not parse the shortcode inside feeds.
		 *
		 */
		if ( ! is_feed() and ( ( is_single() or is_page() ) ) ) {

			//get the table id
			$attributes = shortcode_atts( array(
				'flags-size'                   => '32',
				'orientation'                  => 'horizontal',
				'margin'                       => 5,
				'container-color'              => '#000000',
				'container-opacity'            => '0',
				'container-horizontal-padding' => '0',
				'container-vertical-padding'   => '0',
				'container-border-radius'      => '0'
			), $atts );

			//Return HTML output
			return hreflang_flags(
				$attributes['flags-size'],
				$attributes['orientation'],
				$attributes['margin'],
				$attributes['container-color'],
				$attributes['container-opacity'],
				$attributes['container-horizontal-padding'],
				$attributes['container-vertical-padding'],
				$attributes['container-border-radius'],
                false
			);

		}

	}

	public function wr_public_head() {

		//javascript variables
		echo '<script type="text/javascript">';


		echo 'var hreflangFlags = {';

		//Base URL
		echo 'flagsBaseUrl :"' . esc_js( $this->shared->get( 'url' ) . 'public/assets/img/flags/' ) . '",';

		//General
		echo 'flagsSource :"' . esc_js( get_option( $this->shared->get( 'slug' ) . '_flags_source' ) ) . '",';
		echo 'flagsType :"' . esc_js( get_option( $this->shared->get( 'slug' ) . '_flags_type' ) ) . '",';
		echo 'flagsInitialOpacity :"' . esc_js( get_option( $this->shared->get( 'slug' ) . '_flags_initial_opacity' ) ) . '",';

		//Fixed
		echo 'fixedFlags :"' . esc_js( get_option( $this->shared->get( 'slug' ) . '_fixed_flags' ) ) . '",';
		echo 'fixedFlagsMobile :"' . esc_js( get_option( $this->shared->get( 'slug' ) . '_fixed_flags_mobile' ) ) . '",';
		echo 'fixedFlagsBreakpoint :"' . esc_js( get_option( $this->shared->get( 'slug' ) . '_fixed_flags_breakpoint' ) ) . '",';
		echo 'fixedFlagsPosition :"' . esc_js( get_option( $this->shared->get( 'slug' ) . '_fixed_flags_position' ) ) . '",';
		echo 'fixedFlagsSize :"' . esc_js( get_option( $this->shared->get( 'slug' ) . '_fixed_flags_size' ) ) . '",';
		echo 'fixedFlagsMargin :"' . esc_js( get_option( $this->shared->get( 'slug' ) . '_fixed_flags_margin' ) ) . '",';
		echo 'fixedFlagsContainerColor:"' . esc_js( get_option( $this->shared->get( 'slug' ) . '_fixed_flags_container_color' ) ) . '",';
		echo 'fixedFlagsContainerOpacity:"' . esc_js( get_option( $this->shared->get( 'slug' ) . '_fixed_flags_container_opacity' ) ) . '",';
		echo 'fixedFlagsContainerHorizontalPadding :"' . esc_js( get_option( $this->shared->get( 'slug' ) . '_fixed_flags_container_horizontal_padding' ) ) . '",';
		echo 'fixedFlagsContainerVerticalPadding :"' . esc_js( get_option( $this->shared->get( 'slug' ) . '_fixed_flags_container_vertical_padding' ) ) . '",';
		echo 'fixedFlagsContainerBorderRadius :"' . esc_js( get_option( $this->shared->get( 'slug' ) . '_fixed_flags_container_border_radius' ) ) . '"';

		echo '};';

		//Language
		$language_locale = array();
		foreach ( $this->shared->languages as $iso_code => $name ) {

		    $language_iso_code = get_option( $this->shared->get( 'slug' ) . '_language_' . $iso_code );
		    if(strlen(trim($language_iso_code)) > 0){
                $language_locale[ $iso_code ] = $language_iso_code;
            }

		}
		echo 'hreflangFlags.language = ' . json_encode( $language_locale ) . ';';

		/**
		 * Save as JavaScript object properties the data related to the language alt attribute.
		 *
		 * - The hreflangFlags.languageAltAttribute whether the alt attribute should be implemented or not.
		 * - The hreflangFlags.languageAltAttributePrefix determines the prefix of the alt attribute.
		 * - The hreflangFlags.languageAltAttributeSuffix determines the suffix of the alt attribute.
		 * - The hreflangFlags.languageCodeName is an hash table that includes languages codes and names.
		 */
		if(intval(get_option( $this->shared->get( 'slug' ) . '_language_alt_attribute'), 10) === 1 ){
			echo 'hreflangFlags.languageAltAttribute = "1";';
			echo 'hreflangFlags.languageAltAttributePrefix = "' . esc_js( get_option( $this->shared->get( 'slug' ) . '_language_alt_attribute_prefix' ) ) . '";';
			echo 'hreflangFlags.languageAltAttributeSuffix = "' . esc_js( get_option( $this->shared->get( 'slug' ) . '_language_alt_attribute_suffix' ) ) . '";';
			echo 'hreflangFlags.languageCodeName = ' . json_encode( $this->shared->languages ) . ';';
		}else{
			echo 'hreflangFlags.languageAltAttribute = "0";';
		}

		/**
		 * Save as JavaScript object properties the data related to the locale alt attribute.
		 *
		 * - The hreflangFlags.localeAltAttribute whether the alt attribute should be implemented or not.
		 * - The hreflangFlags.localeAltAttributePrefix determines the prefix of the alt attribute.
		 * - The hreflangFlags.localeAltAttributeSuffix determines the suffix of the alt attribute.
		 * - The hreflangFlags.localeCodeName is an hash table that includes locale codes and names.
		 */
		if(intval(get_option( $this->shared->get( 'slug' ) . '_locale_alt_attribute'), 10) === 1 ){
			echo 'hreflangFlags.localeAltAttribute = "1";';
			echo 'hreflangFlags.localeAltAttributePrefix = "' . esc_js( get_option( $this->shared->get( 'slug' ) . '_locale_alt_attribute_prefix' ) ) . '";';
			echo 'hreflangFlags.localeAltAttributeSuffix = "' . esc_js( get_option( $this->shared->get( 'slug' ) . '_locale_alt_attribute_suffix' ) ) . '";';
			echo 'hreflangFlags.localeCodeName = ' . json_encode( get_option( $this->shared->get( 'slug' ) . '_locale' ) ) . ';';
		}else{
			echo 'hreflangFlags.localeAltAttribute = "0";';
		}

		echo '</script>';

	}

	public function enqueue_styles() {
		wp_enqueue_style( $this->shared->get( 'slug' ) . '-general',
			$this->shared->get( 'url' ) . 'public/assets/css/general.css', array(), $this->shared->get( 'ver' ) );
	}

	public function enqueue_scripts() {

		//Load the 'isMobile' library if the 'Enable Mobile' option is set to 'No'.
		if ( intval( get_option( $this->shared->get( 'slug' ) . '_fixed_flags_mobile' ), 10 ) == 0 ) {
			wp_enqueue_script( $this->shared->get( 'slug' ) . '-is-mobile',
				$this->shared->get( 'url' ) . 'public/assets/inc/isMobile/isMobile.min.js', 'jquery',
				$this->shared->get( 'ver' ), true );
			wp_enqueue_script( $this->shared->get( 'slug' ) . '-general',
				$this->shared->get( 'url' ) . 'public/assets/js/general.js',
				array( 'jquery', $this->shared->get( 'slug' ) . '-is-mobile' ), $this->shared->get( 'ver' ), true );
		} else {
			wp_enqueue_script( $this->shared->get( 'slug' ) . '-general',
				$this->shared->get( 'url' ) . 'public/assets/js/general.js', array('jquery'), $this->shared->get( 'ver' ),
				true );
		}

	}

}