<?php

class Dahf_Widget extends WP_Widget {

	private $shared = null;

	/**
	 * Sets up the widget
	 */
	public function __construct() {

		//assign an instance of the plugin info
		$this->shared = Dahf_Shared::get_instance();

		$widget_ops = array(
			'classname'   => 'dahf-widget',
			'description' => esc_attr__( 'Displays the Hreflang Flags.', 'dahf' ),
		);
		parent::__construct( 'hreflang_flags', 'Hreflang Flags', $widget_ops );

	}

	/**
	 * Outputs the content of the widget
	 *
	 * @param array $args
	 * @param array $instance
	 */
	public function widget( $args, $instance ) {

		echo $args['before_widget'];

		if ( ! empty( $instance['title'] ) ) {
			echo $args['before_title'] . apply_filters( 'widget_title', $instance['title'] ) . $args['after_title'];
		}

		//Generate and echo the HTML output
		hreflang_flags(
			$instance['flags_size'],
			$instance['orientation'],
			$instance['margin'],
			$instance['container_color'],
			$instance['container_opacity'],
			$instance['container_horizontal_padding'],
			$instance['container_vertical_padding'],
			$instance['container_border_radius']
		);

		echo $args['after_widget'];
	}

	/**
	 * Outputs the options form on the Widgets menu
	 *
	 * @param array $instance The widget options
	 */
	public function form( $instance ) {

		$title       = ! empty( $instance['title'] ) ? esc_attr( $instance['title'] ) : '';
		$flags_size  = ! empty( $instance['flags_size'] ) ? esc_attr( $instance['flags_size'] ) : '48';
		$orientation = ! empty( $instance['orientation'] ) ? esc_attr( $instance['orientation'] ) : 'horizontal';

		if ( isset( $instance['margin'] ) and preg_match( $this->shared->digits_regex, $instance['margin'] ) ) {
			$margin = $instance['margin'];
		} else {
			$margin = '8';
		}

		if ( isset( $instance['container_color'] ) and preg_match( $this->shared->hex_rgb_regex,
				$instance['container_color'] )
		) {
			$container_color = $instance['container_color'];
		} else {
			$container_color = '#000000';
		}

		if ( isset( $instance['container_opacity'] ) and preg_match( $this->shared->opacity_regex,
				$instance['container_opacity'] )
		) {
			$container_opacity = $instance['container_opacity'];
		} else {
			$container_opacity = '0';
		}

		if ( isset( $instance['container_horizontal_padding'] ) and preg_match( $this->shared->digits_regex,
				$instance['container_horizontal_padding'] )
		) {
			$container_horizontal_padding = $instance['container_horizontal_padding'];
		} else {
			$container_horizontal_padding = '0';
		}

		if ( isset( $instance['container_vertical_padding'] ) and preg_match( $this->shared->digits_regex,
				$instance['container_vertical_padding'] )
		) {
			$container_vertical_padding = $instance['container_vertical_padding'];
		} else {
			$container_vertical_padding = '0';
		}

		if ( isset( $instance['container_border_radius'] ) and preg_match( $this->shared->digits_regex,
				$instance['container_border_radius'] )
		) {
			$container_border_radius = $instance['container_border_radius'];
		} else {
			$container_border_radius = '0';
		}

		?>

        <!-- Title -->
        <p>
            <label for="<?php echo esc_attr( $this->get_field_id( 'title' ) ); ?>"><?php esc_attr_e( 'Title:',
					'dahf' ); ?></label>
            <input maxlength="255" class="widefat" id="<?php echo esc_attr( $this->get_field_id( 'title' ) ); ?>"
                   name="<?php echo esc_attr( $this->get_field_name( 'title' ) ); ?>" type="text"
                   value="<?php echo esc_attr( $title ); ?>">
        </p>

        <!-- Flags Size -->
        <p>
            <label for="<?php echo esc_attr( $this->get_field_id( 'flags_size' ) ); ?>"><?php esc_attr_e( 'Flags Size:',
					'dahf' ); ?></label>
            <select class="widefat" id="<?php echo esc_attr( $this->get_field_id( 'flags_size' ) ); ?>"
                    name="<?php echo esc_attr( $this->get_field_name( 'flags_size' ) ); ?>">
                <option <?php selected( $flags_size, '16', true ) ?> value="16">16</option>
                <option <?php selected( $flags_size, '24', true ) ?> value="24">24</option>
                <option <?php selected( $flags_size, '32', true ) ?> value="32">32</option>
                <option <?php selected( $flags_size, '48', true ) ?> value="48">48</option>
                <option <?php selected( $flags_size, '64', true ) ?> value="64">64</option>
            </select>
        </p>

        <!-- Orientation -->
        <p>
            <label for="<?php echo esc_attr( $this->get_field_id( 'orientation' ) ); ?>"><?php esc_attr_e( 'Orientation:',
					'dahf' ); ?></label>
            <select class="widefat" id="<?php echo esc_attr( $this->get_field_id( 'orientation' ) ); ?>"
                    name="<?php echo esc_attr( $this->get_field_name( 'orientation' ) ); ?>">
                <option <?php selected( $orientation, 'horizontal', true ) ?>
                        value="horizontal"><?php esc_attr_e( 'Horizontal', 'dahf' ); ?></option>
                <option <?php selected( $orientation, 'vertical', true ); ?>
                        value="vertical"><?php esc_attr_e( 'Vertical', 'dahf' ); ?></option>
            </select>
        </p>

        <!-- Margin -->
        <p>
            <label for="<?php echo esc_attr( $this->get_field_id( 'margin' ) ); ?>"><?php esc_attr_e( 'Margin:',
					'dahf' ); ?></label>
            <input maxlength="4" class="widefat" id="<?php echo esc_attr( $this->get_field_id( 'margin' ) ); ?>"
                   name="<?php echo esc_attr( $this->get_field_name( 'margin' ) ); ?>" type="text"
                   value="<?php echo esc_attr( $margin ); ?>">
        </p>

        <!-- Container Color -->
        <p>
            <label style="display: block;"
                   for="<?php echo esc_attr( $this->get_field_id( 'container_color' ) ); ?>"><?php esc_attr_e( 'Container Color:',
					'dahf' ); ?></label>
            <input maxlength="7" class="widefat" maxlength="7"
                   id="<?php echo esc_attr( $this->get_field_id( 'container_color' ) ); ?>"
                   name="<?php echo esc_attr( $this->get_field_name( 'container_color' ) ); ?>" type="text"
                   value="<?php echo esc_attr( $container_color ); ?>">
        </p>

        <!-- Container Opacity -->
        <p>
            <label for="<?php echo esc_attr( $this->get_field_id( 'container_opacity' ) ); ?>"><?php esc_attr_e( 'Container Opacity:',
					'dahf' ); ?></label>
            <input maxlength="4" class="widefat"
                   id="<?php echo esc_attr( $this->get_field_id( 'container_opacity' ) ); ?>"
                   name="<?php echo esc_attr( $this->get_field_name( 'container_opacity' ) ); ?>" type="text"
                   value="<?php echo esc_attr( $container_opacity ); ?>">
        </p>

        <!-- Container Horizontal Padding -->
        <p>
            <label for="<?php echo esc_attr( $this->get_field_id( 'container_horizontal_padding' ) ); ?>"><?php esc_attr_e( 'Container Horizontal Padding:',
					'dahf' ); ?></label>
            <input maxlength="4" class="widefat"
                   id="<?php echo esc_attr( $this->get_field_id( 'container_horizontal_padding' ) ); ?>"
                   name="<?php echo esc_attr( $this->get_field_name( 'container_horizontal_padding' ) ); ?>" type="text"
                   value="<?php echo esc_attr( $container_horizontal_padding ); ?>">
        </p>

        <!-- Container Vertical Padding -->
        <p>
            <label for="<?php echo esc_attr( $this->get_field_id( 'container_vertical_padding' ) ); ?>"><?php esc_attr_e( 'Container Vertical Padding:',
					'dahf' ); ?></label>
            <input maxlength="4" class="widefat"
                   id="<?php echo esc_attr( $this->get_field_id( 'container_vertical_padding' ) ); ?>"
                   name="<?php echo esc_attr( $this->get_field_name( 'container_vertical_padding' ) ); ?>" type="text"
                   value="<?php echo esc_attr( $container_vertical_padding ); ?>">
        </p>

        <!-- Container Border Radius -->
        <p>
            <label for="<?php echo esc_attr( $this->get_field_id( 'container_border_radius' ) ); ?>"><?php esc_attr_e( 'Container Border Radius:',
					'dahf' ); ?></label>
            <input maxlength="4" class="widefat"
                   id="<?php echo esc_attr( $this->get_field_id( 'container_border_radius' ) ); ?>"
                   name="<?php echo esc_attr( $this->get_field_name( 'container_border_radius' ) ); ?>" type="text"
                   value="<?php echo esc_attr( $container_border_radius ); ?>">
        </p>

		<?php

	}

	/**
	 * Processing widget options on save
	 *
	 * @param array $new_instance The new options
	 * @param array $old_instance The previous options
	 */
	public function update( $new_instance, $old_instance ) {

		$instance = array();

		$instance['title']                        = ! empty( $new_instance['title'] ) ? $new_instance['title'] : $old_instance['title'];
		$instance['flags_size']                   = ! empty( $new_instance['flags_size'] ) ? $new_instance['flags_size'] : $old_instance['flags_size'];
		$instance['orientation']                  = ! empty( $new_instance['orientation'] ) ? $new_instance['orientation'] : $old_instance['orientation'];
		$instance['margin']                       = preg_match( $this->shared->digits_regex,
			$new_instance['margin'] ) ? $new_instance['margin'] : $old_instance['margin'];
		$instance['container_color']              = preg_match( $this->shared->hex_rgb_regex,
			$new_instance['container_color'] ) ? $new_instance['container_color'] : $old_instance['container_color'];
		$instance['container_opacity']            = preg_match( $this->shared->opacity_regex,
			$new_instance['container_opacity'] ) ? $new_instance['container_opacity'] : $old_instance['container_opacity'];
		$instance['container_horizontal_padding'] = preg_match( $this->shared->digits_regex,
			$new_instance['container_horizontal_padding'] ) ? $new_instance['container_horizontal_padding'] : $old_instance['container_horizontal_padding'];
		$instance['container_vertical_padding']   = preg_match( $this->shared->digits_regex,
			$new_instance['container_vertical_padding'] ) ? $new_instance['container_vertical_padding'] : $old_instance['container_vertical_padding'];
		$instance['container_border_radius']      = preg_match( $this->shared->digits_regex,
			$new_instance['container_border_radius'] ) ? $new_instance['container_border_radius'] : $old_instance['container_border_radius'];

		return $instance;
	}
}